unit pProviderCollection;

interface

uses
  SysUtils, Classes, Contnrs, pLogUtils;

type
  TProviderTable = class(TObject)
    Name: String;
    Remote: Integer;
    Ch: String;
  end;

  TProviderCollection = class(TObject)
    constructor Create(LogUtl:TLogUtil; FileName:String);
    destructor  Destroy; override;
  private
    { Private 錾 }
    Log       :TLogUtil;
    FTable    :TObjectList;
    FChList   :TStringList;
    FRemoteList:TStringList;
    FNameList :TStringList;
  public
    { Public 錾 }
    function NameToCh(name:String)       :String;
    function NameToRemote(name:String)   :Integer;
    function ChToName(ch:String)        :String;
    function ChToRemote(ch:String)      :Integer;
    function RemoteToName(rem:Integer)   :String;
    function RemoteToCh(rem:Integer) :String;
    property ChList:TStringList       read FChList;
    property RemoteList:TStringList   read FRemoteList;
    property NameList:TStringList     read FNameList;
  end;


implementation

constructor TProviderCollection.Create(LogUtl:TLogUtil; FileName:String);
var
  F    :TextFile;
  ln   :String;
  p    :Integer;
  Name :String;
  chstr:String;
  Remote:Integer;
  tbl  :TProviderTable;
begin
  Log    := LogUtl;
  FTable := TObjectList.Create;
  FChList     := TStringList.Create;
  FRemoteList := TStringList.Create;
  FNameList   := TStringList.Create;
  try
    AssignFile(F,FileName);
    Reset(F);
    while (not EOF(F)) do
    begin
      Readln(F,ln);
      Trim(ln);
      p := Pos(',',ln);
      if (p>0) then
      begin
        Name := Copy(ln,1,p-1);
        ln := Copy(ln,p+1,Length(ln));
        p := Pos(',',ln);
        if (p>0) then
        begin
          Remote := StrToIntDef(Copy(ln,1,p-1),-1);
          chstr  := Copy(ln,p+1,Length(ln));
          if ((Remote>=0) and (chstr<>'')) then
          begin
            tbl := TProviderTable.Create;
            tbl.Name   := Name;
            tbl.Remote := Remote;
            tbl.Ch     := chstr;
            FTable.Add(tbl);
            FNameList.Add(Name);
            FRemoteList.Add(Format('%.3d',[Remote]));
            FChList.Add(chstr);
          end;
        end;
      end;
    end;
  except
  end;
  //
  FNameList.Sort;
  FRemoteList.Sort;
  FChList.Sort;
  CloseFile(F);
end;

destructor TProviderCollection.Destroy;
begin
  inherited;
  if Assigned(FTable) then FreeAndNil(FTable);
end;

function TProviderCollection.NameToCh(name:String):String;
var
  i   :Integer;
  tbl :TProviderTable;
begin
  Result := '';
  for i:=0 to FTable.Count-1 do
  begin
    tbl := TProviderTable(FTable.Items[i]);
    if (tbl.Name = name) then Result := tbl.Ch;
  end;
end;

function TProviderCollection.NameToRemote(name:String):Integer;
var
  i   :Integer;
  tbl :TProviderTable;
begin
  Result := -1;
  for i:=0 to FTable.Count-1 do
  begin
    tbl := TProviderTable(FTable.Items[i]);
    if (tbl.Name = name) then Result := tbl.Remote;
  end;
end;

function TProviderCollection.ChToName(ch:String):String;
var
  i   :Integer;
  tbl :TProviderTable;
begin
  Result := '-';
  for i:=0 to FTable.Count-1 do
  begin
    tbl := TProviderTable(FTable.Items[i]);
    if (tbl.Ch = ch) then Result := tbl.Name;
  end;
end;

function TProviderCollection.ChToRemote(ch:String):Integer;
var
  i   :Integer;
  tbl :TProviderTable;
begin
  Result := -1;
  for i:=0 to FTable.Count-1 do
  begin
    tbl := TProviderTable(FTable.Items[i]);
    if (tbl.Ch = ch) then Result := tbl.Remote;
  end;
end;

function TProviderCollection.RemoteToName(rem:Integer):String;
var
  i   :Integer;
  tbl :TProviderTable;
begin
  Result := '-';
  for i:=0 to FTable.Count-1 do
  begin
    tbl := TProviderTable(FTable.Items[i]);
    if (tbl.Remote = rem) then Result := tbl.Name;
  end;
end;

function TProviderCollection.RemoteToCh(rem:Integer):String;
var
  i   :Integer;
  tbl :TProviderTable;
begin
  Result := '';
  for i:=0 to FTable.Count-1 do
  begin
    tbl := TProviderTable(FTable.Items[i]);
    if (tbl.Remote = rem) then Result := tbl.Ch;
  end;
end;


end.
