unit pTunerForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActiveX, Winsock, ComCtrls, Contnrs, DateUtils,
  pDriver, B25D_delphi, SegTunerManager_Intf, ExtCtrls, StdCtrls, Taskbar,
  Menus;

const
  // `[iԒ萔
  TUNERSTATE_NOINSTANCE = 0;
  TUNERSTATE_OFF        = 1;
  TUNERSTATE_ON         = 2;
  TUNERSTATE_PLAY       = 3;
  TUNERSTATE_RECORDING  = 4;
  // TListView\ʒu
  TUNER_LISTSUBITEM_STATE   = 0;
  TUNER_LISTSUBITEM_CHANNEL = 1;
  TUNER_LISTSUBITEM_TUNERID = 2;
  // `[iNGXgʒ萔
  TUNER_REQUEST_NONE    = 0;
  TUNER_REQUEST_FILE    = 1;
  TUNER_REQUEST_UDP     = 2;
  // `[idOFFP\Ԓ萔
  TUNER_OFFTIMER_SEC    = 30;

type
  TTunerRequest = class(TObject)
    Handle      :DWORD;
    RequestType :Integer;

    Frequency   :Integer;

    B25Mode     :Boolean;

    MaxTimeOut  :Integer;

    LastUpdate  :TDateTime;

    CreateTime  :TDateTime;

    // File

    Append      :Boolean;

    FileName    :String;

    // UDP

    Address     :String;

    Port        :Integer;

    // 

    FStream     :TFileStream;

    Sock        :TSocket;

    SockAddr    :TSockAddr;
    UDPAddrDat  :DWORD;

  public

    function    SetupFile: Boolean;

    function    SetupUDP: Boolean;

    constructor Create;

    destructor  Destroy; override;

  end;

  THDUSTuner = class(TObject)
  private
    FDLL        :TDriverDLL;
    FHandle     :Pointer;
    FState      :Integer;
    FListItem   :TListItem;
    FID         :String;
    FName       :String;
    FFreq       :Integer;
    FMaxTimeout :Integer;
    FRequests   :TObjectList;
    FLastUpdate :TDateTime;
    FOFFTimer   :TDateTime;
    FB25Decoder :TB25Decoder;
  public
    constructor Create;
    procedure   RefreshListItem;
    procedure   RefreshRequests;
    function    SetRequest(req:TTunerRequest):Boolean;
    procedure   ReceiveTSStream(HDUS:TObject; ABuffer: PByte; ASize: Integer);
    procedure   Suspend;
    procedure   Resume;
    property DLL       :TDriverDLL   read FDLL        write FDLL;
    property Handle    :Pointer      read FHandle     write FHandle;
    property State     :Integer      read FState      write FState;
    property ListItem  :TListItem    read FListItem   write FListItem;
    property ID        :String       read FID         write FID;
    property Name      :String       read FName       write FName;
    property Frequency :Integer      read FFreq       write FFreq;
    property MaxTimeout:Integer      read FMaxTimeout write FMaxTimeout;
    property Requests  :TObjectList  read FRequests   write FRequests;
    property LastUpdate:TDateTime    read FLastUpdate write FLastUpdate;
    property OffTimer  :TDateTime    read FOFFTimer   write FOFFTimer;
    property B25Decoder:TB25Decoder  read FB25Decoder write FB25Decoder;
  end;

  TSegTunerManagerController = class(TForm)
    TunerListView: TListView;
    ProbeTimer: TTimer;
    TrayIcon: TTrayIcon;
    TrayPopupMenu: TPopupMenu;
    O1: TMenuItem;
    X1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure ProbeTimerTimer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure O1Click(Sender: TObject);
    procedure X1Click(Sender: TObject);
  private
    { Private 錾 }
    DriverPath    :String;
    Tuners        :TObjectList;
    CurrentHandle :DWORD;
    function SaveFile(Freq:Integer; Append:Boolean; B25Mode:Boolean; MaxTimeOut:Integer; FileName:String): DWORD;
    function SendUDP(Freq:Integer; B25Mode:Boolean; MaxTimeOut:Integer; Addr:String; Port:Integer): DWORD;
    function Stop(Handle:DWORD): Boolean;
    function UpdateTimeout(Handle:DWORD): Boolean;
    function GetNextHandle: DWORD;
    function SearchTuner(Freq:Integer): THDUSTuner;
    function IsBusy: Boolean;
    function IsRecording: Boolean;
  public
    { Public 錾 }
    procedure WMCopyData(var Msg:TWMCopyData);message WM_COPYDATA;
    procedure WMPowerBroadcast(var Msg:TWMPower);message WM_POWERBROADCAST;
  end;

var
  SegTunerManagerController      : TSegTunerManagerController;
  HDUSDLL    : TDriverDLL;
  WSAData    : TWSAData;

implementation

{$R *.dfm}

procedure SetForceActiveWindow;
var
  hWin: HWND;
  TID, PID, MyID: Integer;
begin
  hWin := GetForegroundWindow;
  if ( hWin = Screen.ActiveForm.Handle ) or ( hWin = 0 ) then Exit;
  MyID := GetCurrentThreadID;
  TID  := GetWindowThreadProcessID( hWin, @PID );
  if AttachThreadInput( TID, MyID, True ) then
  begin
    SetForegroundWindow( Screen.ActiveForm.Handle );
    AttachThreadInput( TID, MyID, False );
  end;
end;

procedure ReceiveTSStreamCallback(HDUS:TObject; ABuffer: PByte; ASize: Integer); stdcall;
var
  i: Integer;
  Tuner: THDUSTuner;
begin
  for i:=0 to SegTunerManagerController.Tuners.Count-1 do
  begin
    Tuner := THDUSTuner(SegTunerManagerController.Tuners[i]);
    if Assigned(Tuner) then
    begin
      if ((Tuner.Handle<>nil) and (Tuner.Handle=HDUS)) then
      begin
        Tuner.ReceiveTSStream(HDUS,ABuffer,ASize);
        break;
      end;
    end;
  end;
end;


// --------------------------------------------------------
//   TTunerRequest
// --------------------------------------------------------

constructor TTunerRequest.Create;
begin
  FStream    := nil;
  UDPAddrDat := 0;
  Sock       := INVALID_SOCKET;
end;

destructor TTunerRequest.Destroy;
begin
  inherited;
  if Assigned(FStream) then FreeAndNil(FStream);
  if ((Sock<>0) and (Sock<>INVALID_SOCKET)) then closesocket(Sock);
end;

function TTunerRequest.SetupFile: Boolean;
begin
  // t@C̃NGXg̓_~[̘^惊NGXgƂ݂Ȃ
  if (FileName<>'') then
  begin
    try
      if (Append) then
      begin
        FStream := TFileStream.Create(FileName,fmOpenReadWrite or fmShareDenyNone);
      end
        else
      begin
        // FileStream̎dlŁAfmCreate̎SharedŊJȂ̂
        // xCreateĂReadWriteŊJȂ
        FStream := TFileStream.Create(FileName,fmCreate or fmShareDenyNone);
        FStream.Free;
        FStream := TFileStream.Create(FileName,fmOpenReadWrite or fmShareDenyNone);
      end;
    except
      if Assigned(FStream) then FreeAndNil(FStream);
      Result := False;
      exit;
    end;
  end;
  Result := True;
end;

function TTunerRequest.SetupUDP: Boolean;
var
  HostEnt: PHostEnt;
  val    : Byte;
  val_i  : Integer;
begin
  Result := True;
  UDPAddrDat := inet_addr(PChar(Address));
  if (UDPAddrDat=$FFFFFFFF) then
  begin
    HostEnt := gethostbyname(PChar(Address));
    if (HostEnt<>nil) then
    begin
      UDPAddrDat := DWORD(HostEnt^.h_addr_list);
    end
      else
    begin
      Result := False;
    end;
  end;
  if ((Sock<>0) and (Sock<>INVALID_SOCKET)) then closesocket(Sock);
  Sock := socket(AF_INET,SOCK_DGRAM, 0);
  FillChar(SockAddr, sizeof(SockAddr), 0);
  SockAddr.sin_family      := AF_INET;
  SockAddr.sin_port        := htons(Port);
  SockAddr.sin_addr.s_addr := UDPAddrDat;
  val := 1;
  setsockopt(Sock, SOL_SOCKET, SO_BROADCAST, @val, SizeOf(val));
  setsockopt(Sock, SOL_SOCKET, SO_KEEPALIVE, @val, SizeOf(val));
  setsockopt(Sock, SOL_SOCKET, SO_REUSEADDR, @val, SizeOf(val));
  val_i := 256*1024*1024;
  setsockopt(Sock, SOL_SOCKET, SO_SNDBUF,  @val_i, SizeOf(val_i));
end;


// --------------------------------------------------------
//   THDUSTuner
// --------------------------------------------------------

constructor THDUSTuner.Create;
begin
  Requests   := TObjectList.Create;
  OffTimer   := 0;
  B25Decoder := TB25Decoder.Create;
  B25Decoder.Initialize(4);
end;

function THDUSTuner.SetRequest(req:TTunerRequest):Boolean;
begin
  // t@CeXg
  if (req.RequestType = TUNER_REQUEST_FILE) then
  begin
    if (not req.SetupFile) then
    begin
      Result := False;
      exit;
    end;
  end;
  // pPbgeXg
  if (req.RequestType = TUNER_REQUEST_UDP) then
  begin
    if (not req.SetupUDP) then
    begin
      Result := False;
      exit;
    end;
  end;
  //
  if (State = TUNERSTATE_NOINSTANCE) then
  begin
    // CX^X쐬
    State  := TUNERSTATE_OFF;
    Handle := DLL.MakeDriverInstance;
    if (Handle=nil) then
    begin
      State := TUNERSTATE_NOINSTANCE;
    end
      else
    begin
      DLL.SetReceiveTSCallback(Handle,ReceiveTSStreamCallback);
    end;
  end;
  if (State = TUNERSTATE_OFF) then
  begin
    // Activate
    if (DLL.Activate(Handle,PChar(ID))) then
    begin
      State := TUNERSTATE_ON;
      Frequency := 0;
    end;
  end;
  if (((State = TUNERSTATE_ON) or (State = TUNERSTATE_PLAY)) and (Frequency <> req.Frequency)) then
  begin
    // Tune
    if (DLL.Tune(Handle,req.Frequency)) then
    begin
      Frequency := req.Frequency;
    end;
  end;

  // Result
  if (((State = TUNERSTATE_ON) or (State = TUNERSTATE_PLAY) or (State = TUNERSTATE_RECORDING))
        and (Frequency = req.Frequency)) then
  begin
    if ((req.RequestType = TUNER_REQUEST_UDP) or (req.RequestType = TUNER_REQUEST_FILE)) then
    begin
      LastUpdate := Now;
      OffTimer   := 0;
      Result     := True;
      Requests.Add(req);
      RefreshRequests;
    end
      else
    begin
      Result := False;
    end;
  end
    else
  begin
    Result := False;
  end;
  RefreshListItem;
end;

procedure THDUSTuner.RefreshRequests;
var
  i: Integer;
  Freq: Integer;
  Req: TTunerRequest;
  Sec,MinSec: Integer;
  Flg: Boolean;
  Updated: Boolean;
  NewState: Integer;
begin
  Updated := False;

  // Ԑ؂̃NGXgE폜
  repeat
  begin
    Flg := True;
    for i:=0 to Requests.Count-1 do
    begin
      Req := TTunerRequest(Requests[i]);
      Sec := SecondsBetween(Req.LastUpdate,Now);
      if (Sec>=Req.MaxTimeOut) then
      begin
        // Ԑ؂ꁨNGXg폜
        Requests.Delete(i);
        Updated := True;
        Flg := False; // Indexς̂ł񃋁[v𔲂ĂȂ
        break;
      end;
    end;
  end until(Flg);

  // NGXg͎cĂH
  if (Requests.Count = 0) then
  begin
    // yȂz
    // StateRECPLAYȂONɖ߂
    if ((State = TUNERSTATE_RECORDING) or (State = TUNERSTATE_PLAY)) then
    begin
      State := TUNERSTATE_ON;
      Updated := True;
    end;
    // ON̏ꍇ(OFF,NOINSTANCEł͂Ȃ)
    if (State = TUNERSTATE_ON) then
    begin
      // OFFɂ
      if (OffTimer = 0) then
      begin
        // ͂߂OFFɂۂ͗P\ԃ^C}Zbg
        OffTimer := Now;
      end
        else
      begin
        // ɗP\ԃ^C}ZbgĂꍇ͎Ԍo߂mF
        if (SecondsBetween(OffTimer,Now)>=TUNER_OFFTIMER_SEC) then
        begin
          // P\ԂÎOFF
          DLL.Deactivate(Handle);
          State    := TUNERSTATE_OFF;
          OffTimer := 0;
          Updated  := True;
        end;
      end;
    end;
  end
    else
  begin
    // yz
    // ЂƂłt@CL^(^)NGXgΘ^撆
    NewState := TUNERSTATE_PLAY;
    for i:=0 to Requests.Count-1 do
    begin
      Req := TTunerRequest(Requests[i]);
      if (Req.RequestType = TUNER_REQUEST_FILE) then NewState := TUNERSTATE_RECORDING;
    end;
    if (State<>NewState) then
    begin
      State   := NewState;
      Updated := True;
    end;

    // ݎg̃NGXg͎cĂH
    Flg := False;
    for i:=0 to Requests.Count-1 do
    begin
      Req := TTunerRequest(Requests[i]);
      if (Frequency = Req.Frequency) then
      begin
        Flg := True;
        break;
      end;
    end;
    if (not Flg) then
    begin
      // cĂȂgύX
      Flg  := False;
      Freq := Frequency;
      // (Ȃ͂)^惊NGXg΂D
      for i:=0 to Requests.Count-1 do
      begin
        Req := TTunerRequest(Requests[i]);
        if (Req.RequestType = TUNER_REQUEST_FILE) then
        begin
          Freq := Req.Frequency;
          Flg  := True;
          break;
        end;
      end;
      // ŌɃNGXĝD
      if (not Flg) then
      begin
        MinSec := $7FFFFFFF;
        for i:=0 to Requests.Count-1 do
        begin
          Req := TTunerRequest(Requests[i]);
          Sec := SecondsBetween(Req.CreateTime,Now);
          if (MinSec>Sec) then
          begin
            Freq := Req.Frequency;
            Flg  := True;
          end;
        end;
      end;
      // gύX
      if (Flg) then
      begin
        DLL.Tune(Handle,Freq);
        Frequency := Freq;
        Updated   := True;
      end;
    end;
  end;
  //
  if (Updated) then
  begin
    RefreshListItem;
  end;
end;

procedure THDUSTuner.RefreshListItem;
var
  str: String;
begin
  if Assigned(ListItem) then
  begin
    ListItem.Caption     := Name;
    case State of
    TUNERSTATE_OFF       : str := 'OFF';
    TUNERSTATE_ON        : str := 'ON';
    TUNERSTATE_PLAY      : str := '';
    TUNERSTATE_RECORDING : str := '^撆';
    else                   str := '------';
    end;
    ListItem.SubItems[TUNER_LISTSUBITEM_STATE] := str;
    //
    if ((FFreq=0) or (State=TUNERSTATE_OFF) or (State=TUNERSTATE_ON)) then
    begin
      str := '-';
    end
      else
    begin
      if (FFreq>=473000) then
      begin
        // UHF
        str := 'UHF'+IntToStr(((FFreq-473000) div 6000)+13)+'ch';
      end
        else
      begin
        // Cable
        str := 'CATV'+IntToStr(((FFreq-225000) div 6000)+23)+'ch';
      end;
    end;
    ListItem.SubItems[TUNER_LISTSUBITEM_CHANNEL] := str;
    //
    ListItem.SubItems[TUNER_LISTSUBITEM_TUNERID] := FID;
  end;
end;

procedure THDUSTuner.ReceiveTSStream(HDUS:TObject; ABuffer: PByte; ASize: Integer);
var
  Req       :TTunerRequest;
  i         :Integer;
  DecBuffer :PByte;
  DecSize   :DWORD;
  NeedB25   :Boolean;
  Decoded   :Boolean;
  buf       :PByte;
  len       :Integer;
begin
  // ̃NGXgB25Decodev̂͂邩H
  NeedB25 := False;
  for i:=0 to Requests.Count-1 do
  begin
    Req := TTunerRequest(Requests[i]);
    if (Req.B25Mode) then
    begin
      NeedB25 := True;
      break;
    end;
  end;
  Decoded := False;
  if (NeedB25) then
  begin
    // ȂB25fR[hs
    if (B25Decoder.DLLReady) and (B25Decoder.Initialized) then
    begin
      B25Decoder.Decode(ABuffer,ASize,DecBuffer,DecSize);
      Decoded := True;
    end;
  end;

  // NGXgɏ
  for i:=0 to Requests.Count-1 do
  begin
    Req := TTunerRequest(Requests[i]);
    buf := ABuffer;
    len := ASize;
    if (Req.B25Mode and Decoded) then
    begin
      Buf := DecBuffer;
      len := DecSize;
    end;
    //
    if (len>0) then
    begin
      // t@Co
      if (Req.RequestType = TUNER_REQUEST_FILE) and (Req.FileName<>'') and Assigned(Req.FStream) then
      begin
        Req.FStream.Write(Buf^,len);
      end;
      // UDPo
      if (Req.RequestType = TUNER_REQUEST_UDP) then
      begin
        sendto(Req.Sock, Buf^, len, 0, Req.SockAddr, SizeOf(Req.SockAddr));
      end;
    end;
  end;
end;

procedure THDUSTuner.Suspend;
begin
  if ((State<>TUNERSTATE_NOINSTANCE) or (State<>TUNERSTATE_OFF)) then
  begin
    DLL.Deactivate(Handle);
  end;
end;

procedure THDUSTuner.Resume;
var
  Req :TTunerRequest;
  i   :Integer;
begin
  for i:=0 to Requests.Count-1 do
  begin
    Req := TTunerRequest(Requests[i]);
    Req.LastUpdate := Now;
  end;
  //
  if ((State<>TUNERSTATE_NOINSTANCE) or (State<>TUNERSTATE_OFF)) then
  begin
    B25Decoder.Reset;
    DLL.Activate(Handle,PChar(ID));
    DLL.Tune(Handle,Frequency);
  end;
end;


// --------------------------------------------------------
//   TSegTunerManagerController
// --------------------------------------------------------

procedure TSegTunerManagerController.FormCreate(Sender: TObject);
var
  i: Integer;
  Tuner: THDUSTuner;
begin
  Randomize;
  DriverPath    := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName));
  Tuners        := TObjectList.Create;
  CurrentHandle := Random($7FFFFFFF);
  HDUSDLL       := TDriverDLL.Create(DriverPath+'SegDriver_hdus.dll');
  for i:=0 to HDUSDLL.Count-1 do
  begin
    if ((Pos('\usb',HDUSDLL.TunerID[i])<>0) and (Pos('SKNET',HDUSDLL.TunerName[i])<>0)) then // MonsterX΍
    begin
      Tuner          := THDUSTuner.Create;
      Tuner.DLL      := HDUSDLL;
      Tuner.Handle   := HDUSDLL.MakeDriverInstance;
      HDUSDLL.SetReceiveTSCallback(Tuner.Handle,ReceiveTSStreamCallback);
      Tuner.State    := TUNERSTATE_OFF;
      Tuner.ID       := HDUSDLL.TunerID[i];
      Tuner.Name     := IntToStr(i)+'-'+HDUSDLL.TunerName[i];
      Tuner.ListItem := TunerListView.Items.Add;
      Tuner.ListItem.Data    := Tuner;
      Tuner.ListItem.SubItems.Add('');
      Tuner.ListItem.SubItems.Add('');
      Tuner.ListItem.SubItems.Add('');
      Tuner.ListItem.SubItems.Add('');
      Tuner.OffTimer := 0;
      Tuner.RefreshListItem;
      Tuners.Add(Tuner);
    end;
  end;
end;

procedure TSegTunerManagerController.ProbeTimerTimer(Sender: TObject);
var
  i: Integer;
  Tuner: THDUSTuner;
begin
  for i:=0 to Tuners.Count-1 do
  begin
    Tuner := THDUSTuner(Tuners[i]);
    if (Tuner.State = TUNERSTATE_NOINSTANCE) then
    begin
      // CX^X쐬
      Tuner.Handle := HDUSDLL.MakeDriverInstance;
      if (Tuner.Handle<>nil) then
      begin
        HDUSDLL.SetReceiveTSCallback(Tuner.Handle,ReceiveTSStreamCallback);
        Tuner.State  := TUNERSTATE_OFF;
        Tuner.RefreshListItem;
        // ЂƂCX^X甲
        break;
      end;
    end
      else
    begin
      // NGXg̐
      Tuner.RefreshRequests;
    end;
  end;
end;

function TSegTunerManagerController.GetNextHandle: DWORD;
begin
  CurrentHandle := CurrentHandle + 1;
  if (CurrentHandle = 0) then CurrentHandle := CurrentHandle + 1;
  Result := CurrentHandle;
end;

function TSegTunerManagerController.SearchTuner(Freq:Integer): THDUSTuner;
var
  Tuner: THDUSTuner;
  i: Integer;
  MinSec,Sec: Integer;
begin
  Result := nil;

  // ғœĝ̂T
  for i:=0 to Tuners.Count-1 do
  begin
    Tuner := THDUSTuner(Tuners[i]);
    if ((Tuner.State=TUNERSTATE_PLAY) or (Tuner.State=TUNERSTATE_RECORDING)) then
    begin
      if (Tuner.Frequency = Freq) then
      begin
        Result := Tuner;
        break;
      end;
    end;
  end;

  // dONœĝ̂T
  if (Result=nil) then
  begin
    for i:=0 to Tuners.Count-1 do
    begin
      Tuner := THDUSTuner(Tuners[i]);
      if (Tuner.State=TUNERSTATE_ON) then
      begin
        if (Tuner.Frequency = Freq) then
        begin
          Result := Tuner;
          break;
        end;
      end;
    end;
  end;

  // dON̂̂T
  if (Result=nil) then
  begin
    for i:=0 to Tuners.Count-1 do
    begin
      Tuner := THDUSTuner(Tuners[i]);
      if (Tuner.State=TUNERSTATE_ON) then
      begin
        Result := Tuner;
        break;
      end;
    end;
  end;

  // dOFF̂̂T
  if (Result=nil) then
  begin
    for i:=0 to Tuners.Count-1 do
    begin
      Tuner := THDUSTuner(Tuners[i]);
      if ((Tuner.State=TUNERSTATE_OFF) or (Tuner.State=TUNERSTATE_NOINSTANCE)) then
      begin
        Result := Tuner;
        break;
      end;
    end;
  end;

  // Đ̂̂T(łTunerPʂōŌRequest)
  MinSec := $7FFFFFFF;
  if (Result=nil) then
  begin
    for i:=0 to Tuners.Count-1 do
    begin
      Tuner := THDUSTuner(Tuners[i]);
      if (Tuner.State=TUNERSTATE_PLAY) then
      begin
        Sec := SecondsBetween(Tuner.LastUpdate,Now);
        if (MinSec>Sec) then
        begin
          Result    := Tuner;
          MinSec    := Sec;
        end;
      end;
    end;
  end;
end;

function TSegTunerManagerController.IsBusy: Boolean;
var
  i: Integer;
  State: Integer;
begin
  Result := False;
  for i:=0 to Tuners.Count-1 do
  begin
    State := THDUSTuner(Tuners[i]).State;
    if ((State = TUNERSTATE_PLAY) or (State = TUNERSTATE_RECORDING)) then Result := True;
  end;
end;

function TSegTunerManagerController.IsRecording: Boolean;
var
  i: Integer;
  State: Integer;
begin
  Result := False;
  for i:=0 to Tuners.Count-1 do
  begin
    State := THDUSTuner(Tuners[i]).State;
    if (State = TUNERSTATE_RECORDING) then Result := True;
  end;
end;

// --------------------------------------------------------
//   CloseQuery
// --------------------------------------------------------

procedure TSegTunerManagerController.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := True;
  //
  if IsRecording then
  begin
    SetForceActiveWindow;
    if (Application.MessageBox('^撆̃`[i[܂Î܂܏IƘ^悪f܂B{ɏI܂H','x',MB_YESNO) <> IDYES) then
    begin
      CanClose := False;
    end;
  end
    else if IsBusy then
  begin
    SetForceActiveWindow;
    if (Application.MessageBox('̃`[i[܂I{ɏI܂H','x',MB_YESNO) <> IDYES) then
    begin
      CanClose := False;
    end;
  end;
end;

procedure TSegTunerManagerController.WMPowerBroadcast(var Msg:TWMPower);
var
  i: Integer;
begin
  if (Msg.PowerEvt = $00000000) then    // PBT_APMQUERYSUSPEND
  begin
    if IsRecording then
    begin
      SetForceActiveWindow;
      if (Application.MessageBox('^撆̃`[i[܂Î܂܏IƘ^悪f܂B{ɏI܂H','x',MB_YESNO) <> IDYES) then
      begin
        Msg.Result := BROADCAST_QUERY_DENY;
      end;
    end;
  end
    else if (Msg.PowerEvt = $00000004) then  // PBT_APMSUSPEND
  begin
    // TXyhɈڍs
    for i:=0 to Tuners.Count-1 do
    begin
      THDUSTuner(Tuners[i]).Suspend;
    end
  end
    else if (Msg.PowerEvt = $00000007) then  // PBT_APMRESUMESUSPEND
  begin
    // TXyh畜A
    for i:=0 to Tuners.Count-1 do
    begin
      THDUSTuner(Tuners[i]).Resume;
    end
  end;
end;


// --------------------------------------------------------
//   SaveFile
// --------------------------------------------------------

function TSegTunerManagerController.SaveFile(Freq:Integer; Append:Boolean; B25Mode:Boolean; MaxTimeOut:Integer; FileName:String): DWORD;
var
  Handle: DWORD;
  Tuner: THDUSTuner;
  Req: TTunerRequest;
  i,j: Integer;
  flg: Boolean;
begin
  Handle := SEGTM_ERROR;

  // t@Cւ̃NGXgȂ
  flg := False;
  for i:=0 to Tuners.Count-1 do
  begin
    Tuner := THDUSTuner(Tuners[i]);
    for j:=0 to Tuner.Requests.Count-1 do
    begin
      Req := TTunerRequest(Tuner.Requests[j]);
      if (Req.RequestType=TUNER_REQUEST_FILE) then
      begin
        if ((Req.FileName = FileName) and (FileName<>'')) then
        begin
          flg := True;
          break;
        end;
      end;
    end;
  end;
  if (not flg) then
  begin
    // Kȃ`[iIAo^
    Tuner  := SearchTuner(Freq);
    if (Tuner<>nil) then
    begin
      Handle := GetNextHandle;
      Req    := TTunerRequest.Create;
      Req.Handle      := Handle;
      Req.RequestType := TUNER_REQUEST_FILE;
      Req.Frequency   := Freq;
      Req.Append      := Append;
      Req.B25Mode     := B25Mode;
      Req.MaxTimeOut  := MaxTimeOut;
      Req.FileName    := FileName;
      Req.LastUpdate  := Now;
      Req.CreateTime  := Now;
      Tuner.SetRequest(Req);
    end;
  end;
  Result := Handle;
end;

// --------------------------------------------------------
//   SendUDP
// --------------------------------------------------------

function TSegTunerManagerController.SendUDP(Freq:Integer; B25Mode:Boolean; MaxTimeOut:Integer; Addr:String; Port:Integer): DWORD;
var
  Handle: DWORD;
  Tuner: THDUSTuner;
  Req: TTunerRequest;
  i,j: Integer;
  flg: Boolean;
begin
  Handle := SEGTM_ERROR;

  // AhXւ̃NGXgȂ
  flg := False;
  for i:=0 to Tuners.Count-1 do
  begin
    Tuner := THDUSTuner(Tuners[i]);
    for j:=0 to Tuner.Requests.Count-1 do
    begin
      Req := TTunerRequest(Tuner.Requests[j]);
      if (Req.RequestType=TUNER_REQUEST_UDP) then
      begin
        if ((Req.Address = Addr) and (Req.Port = Port)) then
        begin
          flg := True;
          break;
        end;
      end;
    end;
  end;
  if (not flg) then
  begin
    // Kȃ`[iIAo^
    Tuner  := SearchTuner(Freq);
    if (Tuner<>nil) then
    begin
      Handle := GetNextHandle;
      Req    := TTunerRequest.Create;
      Req.Handle      := Handle;
      Req.RequestType := TUNER_REQUEST_UDP;
      Req.Frequency   := Freq;
      Req.B25Mode     := B25Mode;
      Req.MaxTimeOut  := MaxTimeOut;
      Req.Address     := Addr;
      Req.Port        := Port;
      Req.LastUpdate  := Now;
      Req.CreateTime  := Now;
      Tuner.SetRequest(Req);
    end;
(*
  end
    else
  begin
    // AhX̃NGXg炻u
    Handle := Req.Handle;
    Req.RequestType := TUNER_REQUEST_UDP;
    Req.B25Mode     := B25Mode;
    Req.MaxTimeOut  := MaxTimeOut;
    Req.LastUpdate  := Now;
    Req.CreateTime  := Now;
    if (Req.Frequency<>Freq) then
    begin
      Req.Frequency := Freq;
      if (HDUSDLL.Tune(Tuner.Handle,Req.Frequency)) then
      begin
        Tuner.Frequency := Req.Frequency;
      end;
    end;
*)
  end;
  Result := Handle;
end;


// --------------------------------------------------------
//   Stop
// --------------------------------------------------------

function TSegTunerManagerController.Stop(Handle:DWORD): Boolean;
var
  i,j: Integer;
  Tuner: THDUSTuner;
  Req: TTunerRequest;
begin
  Result := False;
  Tuner  := nil;
  for i:=0 to Tuners.Count-1 do
  begin
    Tuner := THDUSTuner(Tuners[i]);
    for j:=0 to Tuner.Requests.Count-1 do
    begin
      Req := TTunerRequest(Tuner.Requests[j]);
      if (Req.Handle = Handle) then
      begin
        Tuner.Requests.Delete(j);
        Result := True;
        break;
      end;
    end;
  end;
  Tuner.RefreshRequests;
end;

// --------------------------------------------------------
//   UpdateTimeout
// --------------------------------------------------------

function TSegTunerManagerController.UpdateTimeout(Handle:DWORD): Boolean;
var
  i,j: Integer;
  Tuner: THDUSTuner;
  Req: TTunerRequest;
  Found: Boolean;
begin
  Found := False;
  for i:=0 to Tuners.Count-1 do
  begin
    Tuner := THDUSTuner(Tuners[i]);
    for j:=0 to Tuner.Requests.Count-1 do
    begin
      Req := TTunerRequest(Tuner.Requests[j]);
      if (Req.Handle = Handle) then
      begin
        Found := True;
        Req.LastUpdate := Now;
        break;
      end;
    end;
  end;
  //
  Result := Found;
end;

// --------------------------------------------------------
//   bZ[Wt
// --------------------------------------------------------

procedure TSegTunerManagerController.WMCopyData(var Msg: TWMCopyData);
var
  SaveFileParam: ^TSEGTM_SaveFileParam;
  SendUDPParam:  ^TSEGTM_SendUDPParam;
  HandleParam:   ^TSEGTM_HandleParam;
begin
  case Msg.CopyDataStruct^.dwData of
    SEGTM_ACTIVATEFORM:
      begin
        Application.Restore;
        Show;
        SetForceActiveWindow;
        Msg.Result := SEGTM_OK;
      end;
    SEGTM_SAVEFILE:
      begin
        SaveFileParam := Msg.CopyDataStruct^.lpData;
        Msg.Result := SaveFile( SaveFileParam.Frequency,
                                (SaveFileParam.AppendMode<>0),
                                (SaveFileParam.B25Mode<>0),
                                SaveFileParam.MaxTimeOut,
                                String(SaveFileParam.FileName));
      end;
    SEGTM_SENDUDP:
      begin
        SendUDPParam := Msg.CopyDataStruct^.lpData;
        Msg.Result := SendUDP( SendUDPParam.Frequency,
                                (SendUDPParam.B25Mode<>0),
                                SendUDPParam.MaxTimeOut,
                                String(SendUDPParam.Address),
                                SendUDPParam.Port);
      end;
    SEGTM_STOP:
      begin
        HandleParam := Msg.CopyDataStruct^.lpData;
        if Stop( HandleParam.Handle ) then Msg.Result := SEGTM_OK else Msg.Result := SEGTM_ERROR;
      end;
    SEGTM_UPDATE:
      begin
        HandleParam := Msg.CopyDataStruct^.lpData;
        if UpdateTimeout( HandleParam.Handle ) then Msg.Result := SEGTM_OK else Msg.Result := SEGTM_ERROR;
      end;
    SEGTM_PING:
      begin
        Msg.Result := SEGTM_OK;
      end;
  end;
end;



procedure TSegTunerManagerController.O1Click(Sender: TObject);
begin
  Application.Restore;
end;

procedure TSegTunerManagerController.X1Click(Sender: TObject);
begin
  Close;
end;

Initialization
begin
  CoInitialize(nil);
  WSAStartUp($0101, WSAData);
end;

Finalization
begin
  WSACleanUp;
  CoUninitialize;
end;

end.
