library SegTunerControl;

uses
  SysUtils, Classes, Windows, Messages, DateUtils, SegTunerManager_Intf;

{$R *.res}

function ExecuteProg(App: string): Boolean;
var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
begin
  GetStartupInfo(StartupInfo);
  if CreateProcess(nil,
                   PChar(App),
                   nil,nil,FALSE,0,nil,
                   PChar(GetCurrentDir),
                   StartupInfo,
                   ProcessInfo) then
  begin
    CloseHandle(ProcessInfo.hThread);
    CloseHandle(ProcessInfo.hProcess);
    Result := True;
  end
    else
  begin
    Result := False;
  end;
end;

function ManagerWindowHandle: THandle;
var
  hMutex: THandle;
begin
  hMutex := OpenMutex(MUTEX_ALL_ACCESS, False, SEGTM_MUTEXNAME);
  if (hMutex<>0) then
  begin
    Result := FindWindow(SEGTM_NAME,nil);
  end
    else
  begin
    CloseHandle(hMutex);
    Result := 0;
  end;
end;

function ManagerWindowHandleWithExecute: THandle;
var
  hWnd: THandle;
  StartTime: TDateTime;
begin
  hWnd := ManagerWindowHandle;
  if (hWnd=0) then
  begin
    // N
    if (ExecuteProg(SEGTM_EXENAME)) then
    begin
      StartTime := Now;
      while ((SecondsBetween(StartTime,Now)<SEGTM_EXETIMEOUTSEC) and (hWnd=0)) do
      begin
        hWnd := ManagerWindowHandle;
        if (hWnd=0) then Sleep(100);
      end;
      Result := hWnd;
    end
      else
    begin
      // Ns
      Result := 0;
    end;
  end
    else
  begin
    Result := hWnd;
  end;
end;

function SendToManager(cmd:Integer; ptr:Pointer; siz:Integer):Integer;
var
  hWnd : THandle;
  CDS  : TCopyDataStruct;
begin
  // bZ[WM
  hWnd := ManagerWindowHandleWithExecute;
  if (hWnd=0) then
  begin
    // Ns
    Result := SEGTM_ERROR;
  end
    else
  begin
    // M
    CDS.dwData := cmd; // ŏ畜A
    CDS.cbData := siz;
    CDS.lpData := ptr;
    Result := SendMessage(hWnd, WM_COPYDATA, 0, LParam(@CDS));
  end;
end;

function SegTuner_ChannelToFreq(Ch:WORD; IsCATV:BOOL): DWORD; stdcall;
begin
  if IsCATV then
  begin
    if (Ch>=13) and (Ch<=22) then
    begin
      Result := 111000 + (Ch-13)*6000;
    end
      else if (Ch>=23) and (Ch<=63) then
    begin
      Result := 225000 + (Ch-23)*6000;
    end
      else
    begin
      Result := 0;
    end;
  end
    else
  begin
    if (Ch<13) or (Ch>62) then
    begin
      Result := 0;
    end
      else
    begin
      Result := 473000 + (Ch-13)*6000;
    end;
  end;
end;


function SegTuner_ManagerPing:Boolean; stdcall;
var
  res: Integer;
begin
  res := SendToManager(SEGTM_PING,nil,0);
  if (res=SEGTM_ERROR) then Result:=False else Result:=True;
end;

function SegTuner_ManagerActivate:Boolean; stdcall;
var
  res: Integer;
begin
  res := SendToManager(SEGTM_ACTIVATEFORM,nil,0);
  if (res=SEGTM_ERROR) then Result:=False else Result:=True;
end;

function SegTuner_SaveFile(Freq:DWORD; Append:BOOL; B25Mode:BOOL; MaxTimeOut:DWORD; FileName:PChar; var Handle:DWORD): Boolean; stdcall;
var
  prm: TSEGTM_SaveFileParam;
  res: Integer;
begin
  prm.Frequency  := Freq;
  prm.AppendMode := Word(Append);
  prm.B25Mode    := Word(B25Mode);
  prm.MaxTimeOut := MaxTimeOut;
  StrLCopy(prm.FileName,FileName,sizeof(prm.FileName));
  res := SendToManager(SEGTM_SAVEFILE,@prm,sizeof(prm));
  if (res=SEGTM_ERROR) then Result:=False else Result:=True;
  Handle := res;
end;

function SegTuner_SendUDP(Freq:DWORD; B25Mode:BOOL; MaxTimeOut:DWORD; Addr:PChar; Port:WORD; var Handle:DWORD): Boolean; stdcall;
var
  prm: TSEGTM_SendUDPParam;
  res: Integer;
begin
  prm.Frequency  := Freq;
  prm.B25Mode    := Word(B25Mode);
  prm.MaxTimeOut := MaxTimeOut;
  prm.Port       := Port;
  StrLCopy(prm.Address,Addr,sizeof(prm.Address));
  res := SendToManager(SEGTM_SENDUDP,@prm,sizeof(prm));
  if (res=SEGTM_ERROR) then Result:=False else Result:=True;
  Handle := res;
end;

function SegTuner_Stop(Handle:DWORD): Boolean; stdcall;
var
  prm: TSEGTM_HandleParam;
  res: Integer;
begin
  prm.Handle := Handle;
  res := SendToManager(SEGTM_STOP,@prm,sizeof(prm));
  if (res=SEGTM_ERROR) then Result:=False else Result:=True;
end;

function SegTuner_UpdateTimeout(Handle:DWORD): Boolean; stdcall;
var
  prm: TSEGTM_HandleParam;
  res: Integer;
begin
  prm.Handle := Handle;
  res := SendToManager(SEGTM_UPDATE,@prm,sizeof(prm));
  if (res=SEGTM_ERROR) then Result:=False else Result:=True;
end;

exports
  SegTuner_ChannelToFreq,
  SegTuner_ManagerActivate,
  SegTuner_SaveFile,
  SegTuner_SendUDP,
  SegTuner_Stop,
  SegTuner_UpdateTimeout,
  SegTuner_ManagerPing;

begin
end.
